@@ -217,7 +217,8 @@ g_base64_encode_close (gboolean  break_lines,
  * representation.
  *
  * Return value: a newly allocated, zero-terminated Base-64 encoded
- *               string representing @data.
+ *               string representing @data. The returned string must 
+ *               be freed with g_free().
  *
  * Since: 2.12
  */
@@ -230,16 +231,14 @@ g_base64_encode (const guchar *data,
   gint save = 0;
 
   g_return_val_if_fail (data != NULL, NULL);
-  g_return_val_if_fail (len > 1, NULL);
+  g_return_val_if_fail (len > 0, NULL);
 
   /* We can use a smaller limit here, since we know the saved state is 0 */
   out = g_malloc (len * 4 / 3 + 4);
   outlen = g_base64_encode_step (data, len, FALSE, out, &state, &save);
-  outlen += g_base64_encode_close (FALSE,
-				   out + outlen, 
-				   &state, 
-				   &save);
+  outlen += g_base64_encode_close (FALSE, out + outlen, &state, &save);
   out[outlen] = '\0';
+
   return (gchar *) out;
 }
 
@@ -349,7 +348,8 @@ g_base64_decode_step (const gchar  *in,
  * Decode a sequence of Base-64 encoded text into binary data
  *
  * Return value: a newly allocated buffer containing the binary data
- *               that @text represents
+ *               that @text represents. The returned buffer must
+ *               be freed with g_free().
  *
  * Since: 2.12
  */
@@ -358,19 +358,55 @@ g_base64_decode (const gchar *text,
 		 gsize       *out_len)
 {
   guchar *ret;
-  gint inlen, state = 0;
+  gint input_length, state = 0;
   guint save = 0;
   
   g_return_val_if_fail (text != NULL, NULL);
   g_return_val_if_fail (out_len != NULL, NULL);
 
-  inlen = strlen (text);
-  ret = g_malloc0 (inlen * 3 / 4);
+  input_length = strlen (text);
+
+  g_return_val_if_fail (input_length > 1, NULL);
+
+  ret = g_malloc0 (input_length * 3 / 4);
   
-  *out_len = g_base64_decode_step (text, inlen, ret, &state, &save);
+  *out_len = g_base64_decode_step (text, input_length, ret, &state, &save);
   
   return ret; 
 }
+ 
+/**
+ * g_base64_decode_inplace:
+ * @text: zero-terminated string with base64 text to decode
+ * @out_len: The length of the decoded data is written here
+ *
+ * Decode a sequence of Base-64 encoded text into binary data
+ * by overwriting the input data.
+ *
+ * Return value: The binary data that @text responds. This pointer
+ *               is the same as the input @text.
+ *
+ * Since: 2.20
+ */
+guchar *
+g_base64_decode_inplace (gchar *text,
+                         gsize *out_len)
+{
+  gint input_length, state = 0;
+  guint save = 0;
+  
+  g_return_val_if_fail (text != NULL, NULL);
+  g_return_val_if_fail (out_len != NULL, NULL);
+
+  input_length = strlen (text);
+
+  g_return_val_if_fail (input_length > 1, NULL);
+
+  *out_len = g_base64_decode_step (text, input_length, (guchar *) text, &state, &save);
+  
+  return text; 
+}
+
 
 #define __G_BASE64_C__
 #include "galiasdef.c"
