@@ -122,6 +122,11 @@ g_param_type_init (void)
   };
   GType type;
 
+  /* This should be registred as GParamSpec instead of GParam, for
+   * consistency sake, so that type name can be mapped to struct name,
+   * However, some language bindings, most noticable the python ones
+   * depends on the "GParam" identifier, see #548689
+   */
   type = g_type_register_fundamental (G_TYPE_PARAM, g_intern_static_string ("GParam"), &param_spec_info, &finfo, G_TYPE_FLAG_ABSTRACT);
   g_assert (type == G_TYPE_PARAM);
   g_value_register_transform_func (G_TYPE_PARAM, G_TYPE_PARAM, value_param_transform_value);
@@ -196,7 +201,7 @@ g_param_spec_ref (GParamSpec *pspec)
   g_return_val_if_fail (G_IS_PARAM_SPEC (pspec), NULL);
   g_return_val_if_fail (pspec->ref_count > 0, NULL);
 
-  g_atomic_int_inc (&pspec->ref_count);
+  g_atomic_int_inc ((int *)&pspec->ref_count);
 
   return pspec;
 }
@@ -215,7 +220,7 @@ g_param_spec_unref (GParamSpec *pspec)
   g_return_if_fail (G_IS_PARAM_SPEC (pspec));
   g_return_if_fail (pspec->ref_count > 0);
 
-  is_zero = g_atomic_int_dec_and_test (&pspec->ref_count);
+  is_zero = g_atomic_int_dec_and_test ((int *)&pspec->ref_count);
 
   if (G_UNLIKELY (is_zero))
     {
